#!/usr/bin/env node

import { Server } from "@modelcontextprotocol/sdk/server/index.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";
import {
  CallToolRequestSchema,
  ListToolsRequestSchema,
} from "@modelcontextprotocol/sdk/types.js";
import { google } from "googleapis";
import fs from "fs";
import minimist from "minimist";

const log = (...args) => console.error("[SearchConsoleMCP]", ...args);

class SearchConsoleMCPServer {
  constructor() {
    this.server = new Server(
      {
        name: "search-console-mcp-server",
        version: "1.0.0",
      },
      {
        capabilities: {
          tools: {},
        },
      }
    );
    this.searchConsoleClient = null;
    this.defaultSiteUrl = null;
    this.setupToolHandlers();
  }

  async initialize(args) {
    const keyFilePath = args["key-file"];
    if (!keyFilePath) {
      throw new Error(
        "Key file path is not specified with --key-file argument."
      );
    }

    log(`Reading key file: ${keyFilePath}`);

    let credentials;
    try {
      const fileContent = fs.readFileSync(keyFilePath, "utf8");
      credentials = JSON.parse(fileContent);
    } catch (error) {
      log("Failed to read or parse key file.", error);
      throw new Error(
        `Failed to read key file: ${keyFilePath}. Error: ${error.message}`
      );
    }

    const auth = new google.auth.GoogleAuth({
      credentials,
      scopes: ["https://www.googleapis.com/auth/webmasters.readonly"],
    });

    this.searchConsoleClient = google.searchconsole({ version: "v1", auth });
    this.defaultSiteUrl = args["site-url"] || null;
    log(`Default site URL set to: ${this.defaultSiteUrl}`);
  }

  setupToolHandlers() {
    const tools = [
      {
        name: "list_sites",
        description: "Get a list of accessible Search Console sites.",
        inputSchema: { type: "object", properties: {}, required: [] },
      },
      {
        name: "get_search_analytics",
        description: "Get search analytics data for a specified date range.",
        inputSchema: {
          type: "object",
          properties: {
            siteUrl: {
              type: "string",
              description:
                "Site URL. If omitted, the default value will be used.",
            },
            startDate: {
              type: "string",
              description: "Start date in YYYY-MM-DD format.",
            },
            endDate: {
              type: "string",
              description: "End date in YYYY-MM-DD format.",
            },
            dimensions: {
              type: "array",
              items: { type: "string" },
              description: "Dimensions to group by (e.g., ['query', 'page'])",
            },
            rowLimit: {
              type: "number",
              description: "Maximum number of rows to return (default: 1000)",
            },
          },
          required: ["startDate", "endDate"],
        },
      },
      {
        name: "get_daily_search_report",
        description:
          "Get daily search performance summary for a specified date.",
        inputSchema: {
          type: "object",
          properties: {
            siteUrl: {
              type: "string",
              description:
                "Site URL. If omitted, the default value will be used.",
            },
            date: {
              type: "string",
              description: "Date (YYYY-MM-DD format, defaults to yesterday)",
            },
          },
          required: [],
        },
      },
      {
        name: "get_url_inspection",
        description: "Inspect a specific URL to get its Search Console data.",
        inputSchema: {
          type: "object",
          properties: {
            siteUrl: {
              type: "string",
              description:
                "Site URL. If omitted, the default value will be used.",
            },
            inspectionUrl: {
              type: "string",
              description: "The URL to inspect (must be under the site URL)",
            },
          },
          required: ["inspectionUrl"],
        },
      },
    ];

    this.server.setRequestHandler(ListToolsRequestSchema, async () => ({
      tools,
    }));

    this.server.setRequestHandler(CallToolRequestSchema, async (request) => {
      const { name, arguments: args } = request.params;
      log(`Calling tool: ${name} with args:`, args);
      try {
        let result;
        switch (name) {
          case "list_sites":
            result = await this.listSites();
            break;
          case "get_search_analytics":
            result = await this.getSearchAnalytics(args);
            break;
          case "get_daily_search_report":
            result = await this.getDailySearchReport(args.siteUrl, args.date);
            break;
          case "get_url_inspection":
            result = await this.getUrlInspection(
              args.siteUrl,
              args.inspectionUrl
            );
            break;
          default:
            throw new Error(`Unknown tool: ${name}`);
        }
        return {
          content: [{ type: "text", text: JSON.stringify(result, null, 2) }],
        };
      } catch (error) {
        log("Tool execution error:", error);
        return {
          content: [{ type: "text", text: `Error: ${error.message}` }],
        };
      }
    });
  }

  _getSiteUrl(siteUrl) {
    const url = siteUrl || this.defaultSiteUrl;
    if (!url) {
      throw new Error(
        "Site URL is not specified. Pass it as an argument to the tool or set a default value in the extension settings."
      );
    }
    return url;
  }

  async listSites() {
    log("listSites called.");
    try {
      const response = await this.searchConsoleClient.sites.list();
      const sites = response.data.siteEntry || [];

      log(`Found ${sites.length} sites`);
      return {
        sites: sites.map((site) => ({
          siteUrl: site.siteUrl,
          permissionLevel: site.permissionLevel,
        })),
      };
    } catch (error) {
      log("Error listing sites:", error);
      throw new Error(`Failed to list sites: ${error.message}`);
    }
  }

  async getSearchAnalytics(args) {
    const siteUrl = this._getSiteUrl(args.siteUrl);
    const startDate = args.startDate;
    const endDate = args.endDate;
    const dimensions = args.dimensions || ["query"];
    const rowLimit = args.rowLimit || 1000;

    log(
      `getSearchAnalytics called for site: ${siteUrl}, date range: ${startDate} to ${endDate}`
    );

    try {
      const request = {
        siteUrl,
        requestBody: {
          startDate,
          endDate,
          dimensions,
          rowLimit,
        },
      };

      // googleapisパッケージの正しいメソッド名を使用
      const response = await this.searchConsoleClient.searchanalytics.query(
        request
      );

      const rows = response.data.rows || [];
      const summary = {
        totalClicks: 0,
        totalImpressions: 0,
        averageCtr: 0,
        averagePosition: 0,
      };

      if (rows.length > 0) {
        summary.totalClicks = rows.reduce(
          (sum, row) => sum + (row.clicks || 0),
          0
        );
        summary.totalImpressions = rows.reduce(
          (sum, row) => sum + (row.impressions || 0),
          0
        );
        summary.averageCtr =
          summary.totalImpressions > 0
            ? ((summary.totalClicks / summary.totalImpressions) * 100).toFixed(
                2
              )
            : 0;
        summary.averagePosition =
          rows.reduce((sum, row) => sum + (row.position || 0), 0) / rows.length;
      }

      return {
        siteUrl,
        dateRange: { startDate, endDate },
        summary,
        rows: rows.slice(0, 100), // Limit to first 100 rows for readability
        totalRows: rows.length,
      };
    } catch (error) {
      log("Error getting search analytics:", error);
      throw new Error(`Failed to get search analytics: ${error.message}`);
    }
  }

  async getDailySearchReport(siteUrl, dateStr) {
    const site = this._getSiteUrl(siteUrl);
    const date = dateStr || this._getYesterdayDate();

    log(`getDailySearchReport called for site: ${site}, date: ${date}`);

    try {
      const request = {
        siteUrl: site,
        requestBody: {
          startDate: date,
          endDate: date,
          dimensions: ["query", "page"],
          rowLimit: 1000,
        },
      };

      // googleapisパッケージの正しいメソッド名を使用
      const response = await this.searchConsoleClient.searchanalytics.query(
        request
      );

      const rows = response.data.rows || [];
      const summary = {
        date,
        totalClicks: rows.reduce((sum, row) => sum + (row.clicks || 0), 0),
        totalImpressions: rows.reduce(
          (sum, row) => sum + (row.impressions || 0),
          0
        ),
        averageCtr: 0,
        averagePosition: 0,
        uniqueQueries: new Set(rows.map((row) => row.keys[0])).size,
        uniquePages: new Set(rows.map((row) => row.keys[1])).size,
      };

      if (summary.totalImpressions > 0) {
        summary.averageCtr = (
          (summary.totalClicks / summary.totalImpressions) *
          100
        ).toFixed(2);
      }

      if (rows.length > 0) {
        summary.averagePosition = (
          rows.reduce((sum, row) => sum + (row.position || 0), 0) / rows.length
        ).toFixed(1);
      }

      return {
        siteUrl: site,
        summary,
        topQueries: rows
          .sort((a, b) => (b.clicks || 0) - (a.clicks || 0))
          .slice(0, 10)
          .map((row) => ({
            query: row.keys[0],
            clicks: row.clicks,
            impressions: row.impressions,
            ctr: row.ctr,
            position: row.position,
          })),
      };
    } catch (error) {
      log("Error getting daily search report:", error);
      throw new Error(`Failed to get daily search report: ${error.message}`);
    }
  }

  async getUrlInspection(siteUrl, inspectionUrl) {
    const site = this._getSiteUrl(siteUrl);

    log(`getUrlInspection called for site: ${site}, URL: ${inspectionUrl}`);

    try {
      const request = {
        siteUrl: site,
        requestBody: {
          inspectionUrl,
        },
      };

      // googleapisパッケージの正しいメソッド名を使用
      const response =
        await this.searchConsoleClient.urlInspection.index.inspect(request);

      const inspectionResult = response.data.inspectionResult;
      if (!inspectionResult) {
        throw new Error("No inspection result returned");
      }

      return {
        siteUrl: site,
        inspectionUrl,
        indexStatusResult: inspectionResult.indexStatusResult,
        ampResult: inspectionResult.ampResult,
        mobileUsabilityResult: inspectionResult.mobileUsabilityResult,
        richResultsResult: inspectionResult.richResultsResult,
      };
    } catch (error) {
      log("Error inspecting URL:", error);
      throw new Error(`Failed to inspect URL: ${error.message}`);
    }
  }

  _getYesterdayDate() {
    const yesterday = new Date();
    yesterday.setDate(yesterday.getDate() - 1);
    return yesterday.toISOString().split("T")[0];
  }

  async run() {
    const args = minimist(process.argv.slice(2));
    await this.initialize(args);

    const transport = new StdioServerTransport();
    await this.server.connect(transport);
    log("Search Console MCP Server started");
  }
}

const server = new SearchConsoleMCPServer();
server.run().catch((error) => {
  log("Server error:", error);
  process.exit(1);
});
